@@ -1,11 +1,14 @@
 from __future__ import annotations
 
+import ast
 import asyncio
 import gc
+import inspect
 import warnings
 from contextvars import ContextVar, Token
 from typing import Any, Dict, List, Literal, Optional, Type, Union
 
+from curio.meta import from_coroutine
 from sqlalchemy import create_engine
 from sqlalchemy.engine import Engine
 from sqlalchemy.engine.url import URL
@@ -32,11 +35,13 @@ try:
 except ImportError:
     create_async_engine = None
 
-_session: ContextVar[Dict[str, Session | AsyncSession]] = ContextVar("_session", default={})
+_session: ContextVar[Dict[str, Dict[SQLAlchemy, Session | AsyncSession]]] = ContextVar(
+    "_session", default={"sync": {}, "async": {}}
+)
 
 
 def start_session() -> Token[Dict[str, Session | AsyncSession]]:
-    return _session.set({})
+    return _session.set({"sync": {}, "async": {}})
 
 
 def reset_session(token: Token[Dict[str, Session | AsyncSession]]) -> None:
@@ -52,49 +57,64 @@ class DBSession:
             raise SessionNotInitialisedError
         session = self.db.sync_session_maker(**self.db.sync_session_args)
         session_dict = _session.get()
-        session_dict[self.db] = session
+        session_dict["sync"][self.db] = session
+        _session.set(session_dict)
+
+        return self.db
+
+    def __enter__(self):
+        if not isinstance(self.db.sync_session_maker, sessionmaker):
+            raise SessionNotInitialisedError
+        session = self.db.sync_session_maker(**self.db.sync_session_args)
+        session_dict = _session.get()
+        session_dict["sync"][self.db] = session
         _session.set(session_dict)
 
         return self.db
 
     def __exit__(self, exc_type, exc_value, traceback):
-        sess = self.db.session
         if exc_type is not None:
-            sess.rollback()
+            self.db.sync_session.rollback()
 
         elif self.db.commit_on_exit:
             try:
-                sess.commit()
+                self.db.sync_session.commit()
+                self.db.sync_session.rollback()
             except:
-                sess.rollback()
-        sess.close()
-        del sess
-        session_dict = _session.get()
-        session_dict.pop(self.db)
-        _session.set(session_dict)
+                pass
+        try:
+            self.db.sync_session.close()
+            session_dict = _session.get()
+            session_dict["sync"].pop(self.db)
+            _session.set(session_dict)
+        except:
+            pass
 
     async def __aenter__(self):
         if not isinstance(self.db.async_session_maker, async_sessionmaker):
             raise SessionNotInitialisedError
         session = self.db.async_session_maker(**self.db.async_session_args)
         session_dict = _session.get()
-        session_dict[self.db] = session
+        session_dict["async"][self.db] = session
         _session.set(session_dict)
         return self.db
 
     async def __aexit__(self, exc_type, exc_value, traceback):
-        sess = self.db.session
         if exc_type is not None:
-            await sess.rollback()
+            await self.db.session.rollback()
         elif self.db.commit_on_exit:
             try:
-                await sess.commit()
+                await self.db.session.commit()
+                await self.db.session.rollback()
             except:
-                await sess.rollback()
-        await self.db.session.close()
-        session_dict = _session.get()
-        session_dict.pop(self.db)
-        _session.set(session_dict)
+                pass
+        try:
+            await self.db.session.close()
+            session_dict = _session.get()
+            session_dict["async"].pop(self.db)
+            _session.set(session_dict)
+        except:
+            pass
 
 
 class SQLAlchemy:
@@ -128,7 +148,7 @@ class SQLAlchemy:
         self.engine_args = engine_args or {}
         self.async_engine_args = async_engine_args or {}
         self.sync_session_args = session_args or {}
-        self.async_session_args = async_session_args or {"expire_on_commit": False}
+        self.async_session_args = async_session_args or {}
         self.commit_on_exit = commit_on_exit
         self.session_manager: DBSession = _session_manager
         self.verbose = verbose
@@ -145,6 +165,9 @@ class SQLAlchemy:
         self.async_session_maker: async_sessionmaker = None
         if self.url:
             self.init()
+        self.sync_session_args["expire_on_commit"] = self.async_session_args[
+            "expire_on_commit"
+        ] = False
 
     def init(
         self,
@@ -257,18 +280,23 @@ class SQLAlchemy:
 
     @property
     def session(self) -> Union[Session, AsyncSession]:
-        s: Union[Session, AsyncSession] = _session.get()[self]
-        if not s:
+        sessions = _session.get()
+        if sessions["async"].get(self):
+            return sessions["async"][self]
+        elif sessions["sync"].get(self):
+            return sessions["sync"][self]
+        else:
             raise SessionNotInitialisedError
-        return s
 
     @property
     def sync_session(self) -> Session:
-        s = self.session
-        if self.async_ and type(s) == AsyncSession:
-            return s.sync_session
+        sessions = _session.get()
+        if sessions["sync"].get(self):
+            return sessions["sync"][self]
+        elif sessions["async"].get(self):
+            return sessions["async"][self].sync_session
         else:
-            raise SessionNotAsync
+            raise SessionNotInitialisedError
 
     def _make_dialects(self) -> None:
         self.BigInteger = BigInteger()
